use defmt::{info, warn};
use embassy_net::Stack;

use embassy_time::{Duration, Timer};
use rust_mqtt::{
    client::{client::MqttClient, client_config::ClientConfig},
    utils::rng_generator::CountingRng,
};

#[embassy_executor::task]
pub async fn mqtt_task(stack: Stack<'static>) {
    info!("Connecting...");
    let mut rx_buffer = [0; 128];
    let mut tx_buffer = [0; 128];
    loop {
        let mut socket = embassy_net::tcp::TcpSocket::new(stack, &mut rx_buffer, &mut tx_buffer);
        socket.set_timeout(Some(Duration::from_secs(10)));

        let host_addrs = stack
            .dns_query("test.mosquitto.org", embassy_net::dns::DnsQueryType::A)
            .await
            .unwrap();
        let host_addr = host_addrs.iter().next().unwrap();

        if let Err(e) = socket.connect((*host_addr, 1883)).await {
            warn!("connect error: {:?}", e);
            continue;
        }
        info!("Connected to {:?}", socket.remote_endpoint());

        let mut config = ClientConfig::new(
            rust_mqtt::client::client_config::MqttVersion::MQTTv5,
            CountingRng(20000),
        );
        config.add_max_subscribe_qos(rust_mqtt::packet::v5::publish_packet::QualityOfService::QoS1);
        config.add_client_id("client");
        // config.add_username(USERNAME);
        // config.add_password(PASSWORD);
        config.max_packet_size = 100;
        let mut recv_buffer = [0; 80];
        let mut write_buffer = [0; 80];

        let mut client =
            MqttClient::<_, 5, _>::new(socket, &mut write_buffer, 80, &mut recv_buffer, 80, config);

        client.connect_to_broker().await.unwrap();

        loop {
            client
                .send_message(
                    "/malled",
                    b"hello2",
                    rust_mqtt::packet::v5::publish_packet::QualityOfService::QoS0,
                    true,
                )
                .await
                .unwrap();
            Timer::after_millis(2000).await;
        }
    }
}
